#if defined(_WIN32) || defined(_WIN64)
#define WINVER 0x0600
#define _WIN32_WINNT 0x0600
#endif

#include "test.h"

#include <boost/bind.hpp>
#include <boost/make_shared.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

Service::Service(void)
  : _ioService()
  , _waitForTransfers(true)
{
  _curl.reset(new curl_asio(_ioService));
  _ioServiceWork.reset(new boost::asio::io_service::work(_ioService));
  _thread.reset(new boost::thread(boost::bind(&Service::workerThread, this)));
}

Service::~Service(void)
{
  _ioServiceWork.reset();
  
  if (_waitForTransfers)
  {
    _thread->join();
    _ioService.stop();
  }
  else
  {
    _ioService.stop();
    _thread->join();
  }

  _curl.reset();
}

void Service::workerThread(void)
{
  _ioService.run();
}

void Service::test()
{
  std::string url = "http://www.gnu.org/software/gettext/manual/gettext.html";

  for (size_t i = 0; i < 1; ++i)
  {
    boost::filesystem::path file(boost::lexical_cast<std::string>(i) + ".test");
    _ioService.post(boost::bind(&Service::startTransfer, this, url, file));
  }

   boost::asio::deadline_timer t(_ioService, boost::posix_time::seconds(1));
   t.wait();

   std::vector<curl_asio::transfer::ptr> transfers;
   {
     ScopedLock lock(_mutex);
     transfers.swap(_transfers);
   }
   for (size_t i = 0; i < transfers.size(); ++i)
   {
     // pause after some time
     //_ioService.post(boost::bind(&curl_asio::transfer::pause, transfers[i], true));
   }
}

void Service::startTransfer(std::string & url, boost::filesystem::path & file)
{
  std::vector<curl_asio::transfer::ptr> transfers;
  curl_asio::transfer::ptr transfer = _curl->create_transfer();
  if (transfer)
  {
    boost::shared_ptr<std::ostream> out(new std::ofstream(file.string().c_str()));
    transfer->opt.protocols_ = CURLPROTO_HTTP;
    transfer->on_data_read = boost::bind(&Service::onTransferDataRead, this, out, _1);
    transfer->on_done = boost::bind(&Service::onTransferDone, this, transfer, out, _1);
    transfer->start(url);
    transfers.push_back(transfer);
    // pause right after curl_multi_add_handle()
    //transfer->pause(true);
  }
  if (!transfers.empty())
  {
    ScopedLock lock(_mutex);
    _transfers.swap(transfers);
  }
}

curl_asio::data_action::type Service::onTransferDataRead(boost::shared_ptr<std::ostream> out, const boost::asio::const_buffer& buffer)
{
  out->write(boost::asio::buffer_cast<const char*>(buffer), boost::asio::buffer_size(buffer));
  return curl_asio::data_action::success;
}

void Service::onTransferDone(curl_asio::transfer::ptr transfer, boost::shared_ptr<std::ostream> out, CURLcode result)
{
  if (result == CURLE_OK)
  {
    boost::static_pointer_cast<std::ofstream>(out)->close();
    std::cout << "Transfer of " << transfer->info().effective_url() << " completed successfully (" << transfer->info().total_time() << " seconds)!" << std::endl;
  }
  else
  {
    std::cerr << "Transfer of " << transfer->info().effective_url() << " failed with error " << result << std::endl;
  }
}

int main(int argc, char* argv[])
{
  Service s;
  s.test();

  return 0;
}
